#include <iostream>
#include <fstream>
#include <sstream>
#include "tarloader.h"
#include "common.h"

std::string dataToTsv(const std::vector<eis::DataPoint>& data)
{
	std::stringstream out;
	out<<std::scientific;
	for(const eis::DataPoint& dp : data)
		out<<dp.im.real()<<'\t';
	for(const eis::DataPoint& dp : data)
		out<<0-dp.im.imag()<<'\t';
	return out.str();
}

int main(int argc, char** argv)
{
	if(argc != 5)
	{
		std::cout<<"Usage: "<<argv[0]<<" [TAR_DATASET] [OUT_DIR]\n";
		return 1;
	}


	TarDataset dataset(argv[1], 120);
	if(dataset.size() == 0)
	{
		std::cout<<"could not open "<<argv[1]<<'\n';
		return 2;
	}

	std::filesystem::path outDir(argv[1]);

	if(!checkDir(outDir))
		return 2;

	eis::Spectra spectra = dataset.get(0);
	std::vector<std::fstream> labelFiles;
	for(const std::string& name : spectra.labelNames)
	{
		labelFiles.push_back(std::fstream(outDir/(name+".txt"), std::ios_base::out));
		if(!labelFiles.back().is_open())
		{
			std::cerr<<"Unable to open "<<outDir/(name+".txt")<<" for writeing\n";
			return 2;
		}
	}

	std::fstream spectraFile(outDir/"spectras.txt", std::ios_base::out);
	if(!spectraFile.is_open())
	{
		std::cerr<<"Unable to open "<<(outDir/"spectras.txt")<<" for writeing\n";
		return 2;
	}

	for(size_t i = 0; i < dataset.size(); ++i)
	{
		eis::Spectra spectra = dataset.get(i);
		spectraFile<<dataToTsv(spectra.data)<<'\n';
		if(spectra.labels.size() != labelFiles.size())
		{
			std::cerr<<"not all spectra have the same number of labels\n";
			return 3;
		}
		for(size_t j = 0; j < spectra.labels.size(); ++i)
			labelFiles[j]<<spectra.labels[j]<<'\n';

	}

	spectraFile.close();
	for(std::fstream& file : labelFiles)
		file.close();

	return 0;
}
